﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace HttpExerciser
{
    public class RequestPacketInfo : PacketInfo
    {
        #region Constants

        private const string AuthorizationHeader = "authorization";
        private const string BasicScheme = "basic";

        #endregion

        #region Enums

        public enum PacketVerbs { UNKNOWN = 0, HEAD, GET, PUT, POST, DELETE }

        #endregion

        #region Properties

        public PacketVerbs PacketVerb { get; set; }

        public bool IsProperVerb
        {
            get
            {
                // Compare the desired verb with the verbs we're
                // authorized. Note we're using settings as our 
                // model...
                switch (this.PacketVerb)
                {
                    case PacketVerbs.HEAD:
                        return Properties.Settings.Default.VerbHead;

                    case PacketVerbs.GET:
                        return Properties.Settings.Default.VerbGet;

                    case PacketVerbs.PUT:
                        return Properties.Settings.Default.VerbPut;

                    case PacketVerbs.POST:
                        return Properties.Settings.Default.VerbPost;

                    case PacketVerbs.DELETE:
                        return Properties.Settings.Default.VerbDelete;

                    default:
                        break;
                }

                return false;
            }
        }

        public bool IsProperContentType
        {
            get
            {
                // Compare the desired verb with the verbs we're
                // authorized. Note we're using settings as our 
                // model...
                switch (this.PacketContentType)
                {
                    case PacketContentTypes.Any:
                        return true;

                    case PacketContentTypes.Html:
                        return Properties.Settings.Default.ContentHtml;

                    case PacketContentTypes.Xml:
                        return Properties.Settings.Default.ContentXml;

                    case PacketContentTypes.Atom:
                        return Properties.Settings.Default.ContentAtom;

                    case PacketContentTypes.Json:
                        return Properties.Settings.Default.ContentJson;

                    case PacketContentTypes.UrlEncoded:
                        if (this.PacketVerb == PacketVerbs.UNKNOWN ||
                            this.PacketVerb == PacketVerbs.GET)
                        {
                            return false;
                        }
                        return true;

                    default:
                        break;
                }

                return false;
            }
        }

        public bool ContainsAuthorization
        {
            get
            {
                // Iterate through the headers looking for Authorization
                foreach (string header in base.PacketHeaders)
                {
                    // Check length...if too short, it can't be the header
                    // we want and will cause exception working with 
                    // substring.
                    if (header.Length > AuthorizationHeader.Length)
                    {
                        if (header.Substring(0, AuthorizationHeader.Length).ToLower() == AuthorizationHeader)
                        {
                            return true;
                        }
                    }
                }
                return false;
            }
        }

        public string Username
        {
            get { return this._username; }
        }

        public string Password
        {
            get { return this._password; }
        }

        #endregion

        #region Fields

        // Used to establish critical section for static
        // method processing.
        private static object _verbLock = new object();
        private static object _contentLock = new object();
        private static object _urlLock = new object();

        // Authorization information
        private string _username = String.Empty;
        private string _password = String.Empty;

        #endregion

        #region Constructor

        public RequestPacketInfo(PacketTypes packetType, Int32 packetSize, string packetUri) :
            base(packetType, packetSize, packetUri)
        {
        }

        #endregion

        #region Public methods

        public void ParseAuthorization()
        {
            // Check for Authorization header first
            if (ContainsAuthorization)
            {
                foreach (string header in base.PacketHeaders)
                {
                    // Check length...if too short, it can't be the header
                    // we want and will cause exception working with 
                    // substring.
                    if (header.Length > AuthorizationHeader.Length)
                    {
                        if (header.Substring(0, AuthorizationHeader.Length).ToLower() == AuthorizationHeader)
                        {
                            // Found the Authorization header, now look for the authorization
                            // scheme and credentials.
                            Regex authRegex = new Regex(@":\s*(?<scheme>\w*)\s*(?<credentials>[a-zA-Z0-9=]*)$\s*");

                            // Match regex
                            Match authRegexMatch = authRegex.Match(header.Substring(AuthorizationHeader.Length));

                            // If successful, yank credentials
                            if (authRegexMatch.Success)
                            {
                                // We matched...now check that the scheme is BASIC
                                if (authRegexMatch.Groups["scheme"].Value.ToLower() == BasicScheme)
                                {
                                    // Decode the credentials
                                    string decoded = Encoding.ASCII.GetString(Convert.FromBase64String(authRegexMatch.Groups["credentials"].Value));
                                    string[] values = decoded.Split(':');

                                    // Check for malformed value
                                    if (values.Length == 2)
                                    {
                                        this._username = values[0];
                                        this._password = values[1];
                                        break;
                                    }
                                    else
                                    {
                                        // Invalid/malformed header
                                        throw new BadAuthorizationHeaderException();
                                    }
                                }
                                else
                                {
                                    // Unsupported scheme
                                    throw new UnsupportedSchemeException();
                                }
                            }
                            else
                            {
                                // Invalid/malformed header
                                throw new BadAuthorizationHeaderException();
                            }
                        }
                    }
                }
            }
        }

        #endregion

        #region Static methods

        public static PacketVerbs ParseVerb(string verbCandidate)
        {
            lock (_verbLock)
            {
                // The verb, once converted to upper case, should
                // be parsable by the enumeration. If not, return
                // unknown.
                PacketVerbs verb = PacketVerbs.UNKNOWN;
                try
                {
                    // There is no "TryParse" for enums, so we'll
                    // go for it and handle the exception if there
                    // is an error.
                    verb = (PacketVerbs)Enum.Parse(typeof(PacketVerbs), verbCandidate, true);
                }
                catch
                {
                    // Ignore errors and return UNKNOWN
                }

                return verb;
            }
        }

        public static PacketContentTypes ParseContentType(string contentCandidate)
        {
            lock (_contentLock)
            {
                // The Accept header can be a comma-separated list, so
                // we need to check each one against the values we'll
                // accept. The first one we find, we take.
                PacketContentTypes type = PacketContentTypes.None;
                string[] headerValues = contentCandidate.Split(',');
                if (headerValues.Length == 0)
                {
                    // Accept default
                    type = PacketContentTypes.Any;
                }

                // Check each...
                foreach (string headerValue in headerValues)
                {
                    // Compare the incoming string against the content
                    // types we'll support.
                    if (headerValue == ContentTypeStrings[0])
                    {
                        type = PacketContentTypes.Any;
                        break;
                    }
                    else if (headerValue == ContentTypeStrings[1])
                    {
                        type = PacketContentTypes.Html;
                        break;
                    }
                    else if (headerValue == ContentTypeStrings[2])
                    {
                        type = PacketContentTypes.Xml;
                        break;
                    }
                    else if (headerValue == ContentTypeStrings[3])
                    {
                        type = PacketContentTypes.Atom;
                        break;
                    }
                    else if (headerValue == ContentTypeStrings[4])
                    {
                        type = PacketContentTypes.Json;
                        break;
                    }
                    else if (headerValue == ContentTypeStrings[5])
                    {
                        type = PacketContentTypes.UrlEncoded;
                        break;
                    }
                }

                return type;
            }
        }

        public static PacketContentTypes ParseContentUrl(string urlCandidate)
        {
            lock (_urlLock)
            {
                // Trim any leading '/'
                urlCandidate = urlCandidate.TrimStart('/');

                // Compare the incoming string against the content
                // types we'll support.
                PacketContentTypes type = PacketContentTypes.Any;
                if (urlCandidate.ToLower() == Properties.Settings.Default.UriHtml.ToString().ToLower())
                {
                    type = PacketContentTypes.Html;
                }
                else if (urlCandidate.ToLower() == Properties.Settings.Default.UriXml.ToString().ToLower())
                {
                    type = PacketContentTypes.Xml;
                }
                else if (urlCandidate.ToLower() == Properties.Settings.Default.UriAtom.ToString().ToLower())
                {
                    type = PacketContentTypes.Atom;
                }
                else if (urlCandidate.ToLower() == Properties.Settings.Default.UriJson.ToString().ToLower())
                {
                    type = PacketContentTypes.Json;
                }

                return type;
            }
        }
        #endregion
    }
}
