﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Net;
using System.IO;

namespace HttpExerciser
{
    /// <summary>
    /// Interaction logic for ClientWindow.xaml
    /// </summary>
    public partial class ClientWindow : Window
    {
        List<string> _prefixes = null;
        string _username = String.Empty;
        string _password = String.Empty;
        bool _rememberMe = false;

        public ClientWindow(List<string> prefixes)
        {
            InitializeComponent();

            // Validate
            if (prefixes.Count != cmbContent.Items.Count)
            {
                throw new ApplicationException("The number of prefixes for invoking the service doesn't match the number of options provided to the user.");
            }

            // Save the prefixes
            this._prefixes = prefixes;
        }

        private void cmdSend_Click(object sender, RoutedEventArgs e)
        {
            Int32 attempts = 3;
            while (attempts > 0)
            {
                try
                {
                    // Fabricate the URI
                    string uri = this._prefixes[cmbContent.SelectedIndex];

                    // Check for Authorization header
                    string authorizedHeader = String.Empty;
                    if (!String.IsNullOrEmpty(this._username) && !String.IsNullOrEmpty(this._password))
                    {
                        // Create the header
                        string decoded = String.Format("{0}:{1}", this._username, this._password);
                        string encoded = Convert.ToBase64String(Encoding.ASCII.GetBytes(decoded));
                        authorizedHeader = String.Format("Authorization: BASIC {0}", encoded);
                    }

                    // Determine the content type
                    string contentType = "*/*";
                    string content = String.Empty;
                    if (cmbVerb.SelectedIndex <= 1)
                    {
                        // HEAD/GET, so establish appropriate content type
                        switch (cmbContent.SelectedIndex)
                        {
                            case 0: // negotiated HTML
                                contentType = "text/html";
                                break;

                            case 1: // negotiated XML
                                contentType = "text/xml";
                                break;

                            case 2: // negotiated ATOM
                                contentType = "application/atom+xml";
                                break;

                            case 3: // negotiated JSON
                                contentType = "application/json";
                                break;
                        }
                    }
                    else
                    {
                        // PUT/POST/DELETE, so use encoded text (key/value pairs)
                        contentType = "application/x-www-urlencoded";

                        // Snag the text
                        content = String.Format("Content={0}", tbContent.Text);
                    }

                    // Determine the method
                    string method = "GET"; // selected item #1
                    switch (cmbVerb.SelectedIndex)
                    {
                        case 0: // HEAD
                            method = "HEAD";
                            break;

                        case 2: // PUT
                            method = "PUT";
                            break;

                        case 3: // POST
                            method = "POST";
                            break;

                        case 4: // DELETE
                            method = "DELETE";
                            break;

                        default:
                            break;
                    }

                    // Create the request
                    HttpWebRequest request = WebRequest.Create(uri) as HttpWebRequest;
                    request.Method = method;
                    request.Accept = contentType;
                    if (!String.IsNullOrEmpty(authorizedHeader))
                    {
                        request.Headers.Add(authorizedHeader);
                    }
                    if (cmbVerb.SelectedIndex == 2 || cmbVerb.SelectedIndex == 3)
                    {
                        request.ContentType = contentType;
                        using (StreamWriter wtr = new StreamWriter(request.GetRequestStream()))
                        {
                            wtr.Write(content);
                            wtr.Flush();
                        }
                    }

                    // Call for the response
                    HttpWebResponse response = request.GetResponse() as HttpWebResponse;

                    // Yank the response information
                    lblCode.Content = String.Format("{0} {1}", (Int32)response.StatusCode, response.StatusDescription);

                    // If we made a HEAD call, display the headers, else the content
                    if (cmbVerb.SelectedIndex == 0)
                    {
                        // Build response header output
                        StringBuilder sb = new StringBuilder();
                        sb.AppendFormat("HTTP/{0} {1} {2}\r\n", response.ProtocolVersion, (Int32)response.StatusCode, response.StatusDescription);

                        // Retrieve the response packet headers
                        for (Int32 i = response.Headers.Count - 1; i >= 0; i--)
                        {
                            sb.AppendFormat("{0}: {1}\r\n", response.Headers.GetKey(i), response.Headers[i]);
                        }

                        // Assign text
                        tbResponse.Text = sb.ToString();
                    }
                    else
                    {
                        // Retrieve content
                        string responseContent = String.Empty;
                        using (StreamReader rdr = new StreamReader(response.GetResponseStream()))
                        {
                            responseContent = rdr.ReadToEnd();
                        }

                        tbResponse.Text = responseContent;
                    }

                    // Clear username/password if we're to forget
                    if (!this._rememberMe)
                    {
                        this._username = String.Empty;
                        this._password = String.Empty;
                    }

                    // Exit the loop
                    break;
                }
                catch (WebException ex)
                {
                    // Process normally
                    HttpWebResponse response = ex.Response as HttpWebResponse;
                    string responseContent = String.Empty;
                    using (StreamReader rdr = new StreamReader(response.GetResponseStream()))
                    {
                        responseContent = rdr.ReadToEnd();
                    }
                    tbResponse.Text = responseContent.Length > 0 ? responseContent : ex.ToString();
                    lblCode.Content = String.Format("{0} {1}", (Int32)response.StatusCode, response.StatusDescription);

                    // Check for authentication
                    if (response.StatusCode == HttpStatusCode.Unauthorized)
                    {
                        // Pop up new username/password dialog box
                        ClientAuthenticationWindow authWindow = new ClientAuthenticationWindow();
                        authWindow.Owner = this;
                        bool? result = authWindow.ShowDialog();
                        if (result == true)
                        {
                            // Save username/password...
                            this._username = authWindow.Username;
                            this._password = authWindow.Password;

                            // Save "remember me" setting
                            this._rememberMe = authWindow.RememberMeChecked;
                        }
                        else
                        {
                            // User canceled...
                            break;
                        }
                    }
                    else
                    {
                        // Pop up a messagebox showing the error...
                        MessageBox.Show(ex.Message, "Error Invoking RESTful Service", MessageBoxButton.OK, MessageBoxImage.Error);
                        tbResponse.Text = ex.ToString();
                        break;
                    }
                }

                // Decrease attempt number
                --attempts;
            }
        }

        private void cmbVerb_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Handle only after initial window startup
            if (lblContent != null && tbContent != null)
            {
                // Disable the content textbox for HEAD and GET
                if (cmbVerb.SelectedIndex <= 1)
                {
                    lblContent.IsEnabled = false;
                    tbContent.IsEnabled = false;
                }
                else
                {
                    lblContent.IsEnabled = true;
                    tbContent.IsEnabled = true;
                }
            }
        }

        private void cmdExit_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
