﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Security;
using System.Security.Principal;
using System.Net;
using System.Threading;

namespace HttpExerciser
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        // Declare a delegate used to update the listview when HTTP packets are
        // processed. This is standard WPF multithreaded UI update stuff.
        private delegate void PacketInsertionDelegate(RequestPacketInfo requestPacket, ResponsePacketInfo responsePacket);

        // Background worker that will host the HttpListener
        BackgroundWorker _workerServer = null;

        // Background thread that will host the client...we can't use a BackgroundWorker
        // here because we need an STA thread...
        Thread _clientThread = null;

        // The class that encapsulates the HttpListener workflow
        ListenerWorkUnit _workUnit = null;

        public Window1()
        {
            // Check to see that we're running as administrator (necessary
            // for Vista/Server 2008 to execute HttpListener operations).
            WindowsPrincipal user = new WindowsPrincipal(WindowsIdentity.GetCurrent());
            if (!user.IsInRole("Administrators"))
            {
                MessageBox.Show("This application must be executed by administrators only. The application will shut down now.",
                    "Insufficient Permission", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                Application.Current.Shutdown();
                return;
            }

            // Check to see that HttpListener is supported on this system
            // (it won't be on systems earlier than XP SP2/Server 2003).
            if (!HttpListener.IsSupported)
            {
                MessageBox.Show("This application is not supported by the version of Windows in use. The application will shut down now.",
                    "Not Supported", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                Application.Current.Shutdown();
                return;
            }

            // Continue with initialization normally
            InitializeComponent();
        }

        private void activityList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Yank packet and display
            PacketInfo info = (e.Source as ListView).SelectedItem as PacketInfo;
            if (info != null)
            {
                // Stuff contents into text control
                StringBuilder sb = new StringBuilder();

                // Write headers
                foreach (string header in info.PacketHeaders)
                {
                    sb.AppendFormat("{0}\r\n", header);
                }

                // Append a <cr><lf>
                sb.Append("<cr><lf>\r\n");

                // Append contents
                sb.AppendFormat(!String.IsNullOrEmpty(info.PacketContents) ? info.PacketContents : "<No contents to display>");
                tbContents.Text = sb.ToString();
            }
            else
            {
                tbContents.Text = String.Empty;
            }
        }

        private void cmdExit_Click(object sender, RoutedEventArgs e)
        {
            // If we're listening, stop
            if (this._workerServer != null)
            {
                // Cancel listening operations
                if (this._workUnit != null)
                {
                    this._workUnit.CancelProcessing();
                }

                // Cancel the worker
                this._workerServer.CancelAsync();
            }

            // If we have a client showing, stop
            if (this._clientThread != null)
            {
                // Cancel the worker
                this._clientThread.Abort();
            }

            // Exit the app
            this.Close();
        }

        private void cmdProcess_Click(object sender, RoutedEventArgs e)
        {
            if (this._workerServer != null)
            {
                // Cancel listening operations
                if (this._workUnit != null)
                {
                    this._workUnit.PacketsProcessed -=
                        new ListenerWorkUnit.PacketsProcessedDelegate(workUnit_PacketsProcessed);
                    this._workUnit.CancelProcessing();
                }

                // Cancel the worker
                this._workerServer.CancelAsync();

                // Set the worker and work unit to null
                this._workerServer = null;
                this._workUnit = null;

                // If we have a client showing, stop
                if (this._clientThread != null)
                {
                    // Cancel the worker
                    this._clientThread.Abort();
                    this._clientThread = null;
                }

                // Update button text
                this.cmdProcess.Content = "Start";

                // Update activity label
                lblStatus.Foreground = Brushes.Red;
                lblStatus.Content = "(Inactive)";

                // Handle other general control state
                HandleControlState(true);
            }
            else
            {
                // Initiate listening operations...
                activityList.Items.Clear();
                tbContents.Text = String.Empty;

                // Update button text
                this.cmdProcess.Content = "Stop";

                // Update activity label
                lblStatus.Foreground = Brushes.Blue;
                lblStatus.Content = "(Processing requests on port 8080)";

                // Handle other general control state
                HandleControlState(false);

                // Add the appropriate prefixes
                List<string> prefixes = CreateServerPrefixList();

                // Create a work unit
                this._workUnit = new ListenerWorkUnit(prefixes);
                this._workUnit.PacketsProcessed +=
                    new ListenerWorkUnit.PacketsProcessedDelegate(workUnit_PacketsProcessed);

                // Activate...
                this._workerServer = new BackgroundWorker();
                this._workerServer.WorkerSupportsCancellation = true;
                this._workerServer.DoWork += new DoWorkEventHandler(worker_DoWork);

                // Hand unit to the worker thread
                this._workerServer.RunWorkerAsync(this._workUnit);
            }
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            // Get the BackgroundWorker that raised this event.
            BackgroundWorker worker = sender as BackgroundWorker;

            ListenerWorkUnit workUnit = e.Argument as ListenerWorkUnit;
            workUnit.ProcessPackets(Properties.Settings.Default.Port);
            e.Result = 0;
        }

        void workUnit_PacketsProcessed(RequestPacketInfo requestPacket, ResponsePacketInfo responsePacket)
        {
            // The listener received and processed an HTTP request/response pair and
            // is notifying us that we can now insert the information into the user
            // interface. Since this is a multithreaded application, we'll need to
            // schedule the insertion with the listview's dispatcher.
            activityList.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal,
                new PacketInsertionDelegate(InsertListedPackets),
                requestPacket, responsePacket);
        }

        private void InsertListedPackets(RequestPacketInfo requestPacket, ResponsePacketInfo responsePacket)
        {
            // Insert the processed packets into the listview for user
            // interaction...
            requestPacket.PacketID = activityList.Items.Add(requestPacket);
            responsePacket.PacketID = activityList.Items.Add(responsePacket);        
        }

        private void cmdEditContent_Click(object sender, RoutedEventArgs e)
        {
            ContentEditWindow resourceEditor = new ContentEditWindow();
            bool? result = resourceEditor.ShowDialog();
            if (result == true)
            {
                // Save resource text setting...
                Properties.Settings.Default.Save();
            }
        }

        private void cmdGenClient_Click(object sender, RoutedEventArgs e)
        {
            if (this._clientThread != null)
            {
                // May have been aborted already...
                if (this._clientThread.ThreadState != ThreadState.Running)
                {
                    // Clear the thread
                    this._clientThread.Abort();
                    this._clientThread = null;
                }
                else
                {
                    // Thread is running fine...
                    return;
                }
            }

            if (this._clientThread == null)
            {
                // Activate...
                ThreadStart start = new ThreadStart(clientThread_DoWork);
                this._clientThread = new Thread(start);
                this._clientThread.SetApartmentState(ApartmentState.STA);
                this._clientThread.Start();
            }
        }

        void clientThread_DoWork()
        {
            // Add the appropriate prefixes
            List<string> prefixes = CreateClientPrefixList();

            ClientWorker client = new ClientWorker(prefixes);
            client.ShowClient();
        }

        private void cmdEditSettings_Click(object sender, RoutedEventArgs e)
        {
            SettingsEditWindow settingsEditor = new SettingsEditWindow();
            bool? result = settingsEditor.ShowDialog();
            if (result == true)
            {
                // Save resource text setting...
                Properties.Settings.Default.Save();
            }
        }

        private void Window_Closing(object sender, CancelEventArgs e)
        {
            // Save resource text setting...
            Properties.Settings.Default.Save();
        }

        private void chkSecure_Checked(object sender, RoutedEventArgs e)
        {
            // Save resource text setting...
            Properties.Settings.Default.Save();
        }
    }
}
