﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Net;
using System.Runtime.Serialization;
using System.Threading;
using System.Windows.Forms;
using PhotoShared;

namespace WinFormPhotoClient
{
    public partial class ImageView : UserControl
    {
        public ImageView()
        {
            InitializeComponent();
            ImageDocument = new List<ImageItem>();
        }

        ManualResetEvent _messagingEvent = null;

        ManualResetEvent MessagingEvent
        {
            get
            {
                if (_messagingEvent == null)
                {
                    _messagingEvent = new ManualResetEvent(false);
                }
                return _messagingEvent;
            }
        }

        public event EventHandler<EventArgs> InvalidCredentials;

        bool SelectedDataChanged { get; set; }

        public string LoggedInUserId { get; set; }

        public string CurrentUserId { get; set; }

        //Guid LastImageId { get; set; }

        List<ImageItem> ImageDocument { get; set; }

        private void ImageView_Load(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.LoadMyImages)
            {
                CurrentUserId = LoggedInUserId;
                btnLoadMyPictures_Click(sender, e);
            }
        }

        private void btnLoadFriend_Click(object sender, EventArgs e)
        {
            CurrentUserId = txtFriend.Text.Trim();
            LoadImages();
        }

        void LoadImages()
        {
            Uri imagesUri = null;
            lblNoImages.Visible = false;
            lstImages.Items.Clear();
            imageList.Images.Clear();
            imagesUri = Utility.CreateUri(ServiceType.Image,
                string.Format("Images/{0}", CurrentUserId));

            // Update the UI. If the user isn't the owner of the images,
            // disable the editable fields.
            SetDescriptionFields(!string.Equals(CurrentUserId, LoggedInUserId,
                StringComparison.OrdinalIgnoreCase));

            // Get the list of images.
            WebRequest request = Utility.CreateRequest(imagesUri);
            request.BeginGetResponse(new AsyncCallback(ImagesResponse), request);
        }

        void SetDescriptionFields(bool readOnly)
        {
            chkPublic.Enabled = !readOnly;
            btnDelete.Visible = !readOnly;
            txtCaption.ReadOnly = readOnly;
            txtDescription.ReadOnly = readOnly;
        }

        void ImagesResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                DataContractSerializer serializer = new DataContractSerializer(typeof(ImageItem[]));
                ImageDocument = new List<ImageItem>((ImageItem[])serializer.ReadObject(response.GetResponseStream()));
                response.Close();
                foreach (ImageItem imageItem in ImageDocument)
                {
                    WebRequest imageRequest = Utility.CreateRequest(new Uri(imageItem.ImageUrl));
                    imageRequest.BeginGetResponse(ImageResponse, new object[] { imageRequest, imageItem });
                }
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                switch (response.StatusCode)
                {
                    case HttpStatusCode.Unauthorized:
                        Invoke((MethodInvoker)delegate
                        {
                            MessageBox.Show(this, "Your credentials are incorrect.", "Incorrect credentials", MessageBoxButtons.OK,
                                MessageBoxIcon.Stop);
                            if (InvalidCredentials != null)
                            {
                                InvalidCredentials(this, EventArgs.Empty);
                            }
                        });
                        break;
                    case HttpStatusCode.NotFound:
                        lblNoImages.Visible = true;
                        break;
                }
                response.Close();
            }
        }

        public void ImageResponse(IAsyncResult result)
        {
            object[] values = (object[])(result.AsyncState);
            WebRequest request = (WebRequest)values[0];
            ImageItem imageItem = (ImageItem)values[1];
            try
            {
                WebResponse response = request.EndGetResponse(result);
                Image image = Image.FromStream(response.GetResponseStream());
                this.Invoke((MethodInvoker)delegate
                {
                    lstImages.Items.Add(new ListViewItem(imageItem.Caption,
                        AddImageToList(image)) { Tag = imageItem });
                    if (lstImages.Items.Count == 1)
                    {
                        imageItemBindingSource.DataSource = lstImages.Items[0].Tag;
                        lstImages.Items[0].Selected = true;
                    }
                });
                response.Close();
            }
            catch (Exception we)
            {
                this.Invoke((MethodInvoker)delegate
                {
                    MessageBox.Show(we.ToString());
                });
            }
        }

        private void btnLoadMyPictures_Click(object sender, EventArgs e)
        {
            CurrentUserId = LoggedInUserId;
            txtFriend.Text = string.Empty;
            LoadImages();
        }

        void AddImages(object data)
        {
            // Cast the data to a list of file names.
            string[] filenames = (string[])data;

            // Create a serializer.
            DataContractSerializer itemSerializer =
                new DataContractSerializer(typeof(ImageItem));
            foreach (string filename in filenames)
            {
                // If the string doesn't map to an existing filename,
                // go the next string in the list.
                if (!File.Exists(filename))
                {
                    continue;
                }

                // Create the URI for the service via the helper
                Uri callUri = Utility.CreateUri(ServiceType.Image, "AddImage/");


                WebRequest request = Utility.CreateRequest(callUri);
                request.ContentType = "application/xml";
                request.Method = "POST";
                Stream requestStream = request.GetRequestStream();
                ImageItem imageItem = new ImageItem()
                {
                    PublicImage = false,
                    ImageBytes = File.ReadAllBytes(filename),
                    UserName = LoggedInUserId
                };
                itemSerializer.WriteObject(requestStream, imageItem);
                requestStream.Close();
                IAsyncResult result =
                    request.BeginGetResponse(ReadResponse, new object[] {request, imageItem});
            }
            this.Invoke((MethodInvoker)delegate
            {
                // Enable the control.
                this.Enabled = true;
            });
        }

        void UpdateImage(object data)
        {
            if (imageList.Images.Count == ImageDocument.Count)
            {
                ImageItem imageItem = (ImageItem)data;
                if (data == null)
                {
                    return;
                }
                DataContractSerializer itemSerializer = new DataContractSerializer(typeof(ImageItem));
                Uri callUri = Utility.CreateUri(ServiceType.Image, string.Format("Image/{0}",
                    imageItem.ImageId));
                WebRequest request = Utility.CreateRequest(callUri);
                request.ContentType = "application/xml";
                request.Method = "PUT";
                Stream requestStream = request.GetRequestStream();
                itemSerializer.WriteObject(requestStream, imageItem);
                requestStream.Close();
                request.BeginGetResponse(UpdateResponse, request);
            }
        }

        void DeleteImage(object data)
        {
            ImageItem imageItem = (ImageItem)data;
            if (data == null)
            {
                return;
            }

            // Set the URI
            Uri callUri = Utility.CreateUri(ServiceType.Image, string.Format("Image/{0}",
                imageItem.ImageId));

            // Execute the request.
            WebRequest request = Utility.CreateRequest(callUri);
            request.ContentType = "application/xml";
            request.Method = "DELETE";
            request.BeginGetResponse(DeleteResponse, request);
        }

        void DeleteResponse(IAsyncResult result)
        {
            try
            {
                WebRequest request = (WebRequest)result.AsyncState;
                request.EndGetResponse(result).Close(); // nothing to read, so just complete the request.
            }
            catch (WebException)
            {
                Invoke((MethodInvoker)delegate
                {
                    MessageBox.Show(this, "Failed to delete an image. Try logging back in again.", "Delete Failed", MessageBoxButtons.OK, MessageBoxIcon.Error);
                });
            }
        }

        private int AddImageToList(Image image)
        {
            // This actually assumes that Height==Width. If this stops 
            // being true revisit this code.
            double newHeight = imageList.ImageSize.Height;
            double newWidth = imageList.ImageSize.Width;
            double existingHeight = image.Height;
            double existingWidth = image.Width;
            if (image.Height > image.Width)
            {
                newWidth = (newHeight / existingHeight) * existingWidth;
            }
            else
            {
                newHeight = (newWidth / existingWidth) * existingHeight;
            }
            Bitmap original = new Bitmap(image, new Size((int)newWidth, (int)newHeight));
            Bitmap listBitmap = new Bitmap(imageList.ImageSize.Height, imageList.ImageSize.Width);
            Graphics gfx = Graphics.FromImage(listBitmap);
            gfx.DrawImage(original, 0, 0);
            gfx.Flush();
            imageList.Images.Add(listBitmap);
            return imageList.Images.Count - 1;
        }

        void UpdateResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                request.EndGetResponse(result).Close();
            }
            catch (WebException we)
            {
                Debug.WriteLine(we.Message);
            }
        }

        void ReadResponse(IAsyncResult result)
        {
            DataContractSerializer guidSerializer = new DataContractSerializer(typeof(Guid));
            object[] values = (object[])result.AsyncState;
            WebRequest request = (WebRequest)values[0];
            ImageItem imageItem = (ImageItem)values[1];

            try
            {
                WebResponse response = request.EndGetResponse(result);
                Guid lastImageId = (Guid)guidSerializer.ReadObject(response.GetResponseStream());
                response.Close();
                // Set the ID
                if (lastImageId != Guid.Empty)
                {
                    imageItem.ImageId = lastImageId;
                    ImageDocument.Add(imageItem);
                    this.Invoke((MethodInvoker)delegate
                    {
                        lstImages.Items.Add(new ListViewItem(imageItem.Caption,
                            AddImageToList(imageItem.Image)) { Tag = imageItem });
                    });
                }
            }
            catch (WebException we)
            {
                Debug.WriteLine(we.Message);
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (ofdOpenFile.ShowDialog(this) == DialogResult.OK)
            {
                this.Enabled = false;
                this.lblNoImages.Visible = false;
                ThreadPool.QueueUserWorkItem(new WaitCallback(AddImages), ofdOpenFile.FileNames);
            }
        }

        private void lstImages_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstImages.SelectedIndices.Count > 0)
            {
                lstImages.Select();
                imageItemBindingSource_CurrentItemChanged(sender, e);
                imageItemBindingSource.DataSource = lstImages.SelectedItems[0].Tag;
                imageItemBindingSource.ResetBindings(false);
            }
        }

        private void imageItemBindingSource_CurrentItemChanged(object sender, EventArgs e)
        {
            ImageItem data = imageItemBindingSource.DataSource as ImageItem;
            if (data != null && data.UserName == CurrentUserId)
            {
                foreach (ListViewItem item in lstImages.Items)
                {
                    ImageItem itemData = (ImageItem)item.Tag;
                    if (Guid.Equals(itemData.ImageId, data.ImageId))
                    {
                        item.Text = data.Caption;
                        break;
                    }
                }
                ThreadPool.QueueUserWorkItem(new WaitCallback(UpdateImage), data);
            }
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            for (int i = lstImages.SelectedItems.Count - 1; i >= 0; --i)
            {
                ThreadPool.QueueUserWorkItem(new WaitCallback(DeleteImage),
                    lstImages.SelectedItems[i].Tag);
                lstImages.Items.Remove(lstImages.SelectedItems[i]);
            }
        }
    }
}
