﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using System.Web.UI;
using System.IO;
using System.Text;

using AspNetMvcRestService.Security;

namespace AspNetMvcRestService.Models
{
    public partial class CodeXRCGetService
    {
        private void RenderHtml(ControllerContext context)
        {
            // Pull the user's ID. This works here since we're using the SQL-based
            // membership provider.
            MembershipUser user = Membership.GetUser(context.HttpContext.User.Identity.Name);
            Guid ownerID = (Guid)user.ProviderUserKey;

            context.HttpContext.Response.ContentType = "text/html";
            switch (_directives.Count())
            {
                case 0:
                default:
                    // List all projects for user:
                    // http://{host}/{vdir}/CodeXRC
                    RenderHtmlProjectList(context, ownerID);
                    break;

                case 1:
                    // List specific project for user:
                    // http://{host}/{vdir}/CodeXRC/{project}
                    RenderHtmlProject(context, ownerID, _directives[0]);
                    break;

                case 2:
                    // List specific folder for user:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}
                    RenderHtmlFolder(context, ownerID, _directives[0], _directives[1]);
                    break;

                case 3:
                    // List files by extension for user:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}/{file extension}
                    RenderHtmlFileByType(context, ownerID, _directives[0], _directives[1], _directives[2]);
                    break;

                case 4:
                    // Return specific file to user:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}/{file extension}/{file name}
                    RenderHtmlFileByName(context, ownerID, _directives[0], _directives[1], _directives[2], _directives[3]);
                    break;
            }

            // Assign the content length, but only if HEAD.
            Stream stream = context.HttpContext.Items["OutputStream"] as Stream;
            if (stream != null && stream.CanSeek)
            {
                context.HttpContext.Response.Headers["Content-Length"] = _contentLength.ToString();
            }
        }

        // NOTE: You could use XSLT for this as well. It was just quicker for me to fat-finger
        // this procedural code.
        private void RenderHtmlProjectList(ControllerContext context, Guid ownerID)
        {
            // Pull the project list
            List<ProjectDTO> projectList = _dal.ListProjects(context, ownerID);
            if (projectList != null && projectList.Count() > 0)
            {
                using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
                {
                    StringBuilder sb = new StringBuilder();
                    sb.Append("<html><head><title>CodeXRC</title></head><body>");
                    sb.Append("Your projects:<br/><ul>");
                    foreach (ProjectDTO item in projectList)
                    {
                        sb.AppendFormat("<li><a href='{0}'>{1}</a></li>", item.Link, item.DisplayName);
                    }
                    sb.Append("</ul></body><html>");
                    wtr.WriteLine(sb.ToString());
                    wtr.Flush();
                    _contentLength = GetContentLength(context);
                    wtr.Close();
                }
            }
            else
            {
                // No projects associated with user
                using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
                {
                    StringBuilder sb = new StringBuilder();
                    sb.Append("<html><head><title>CodeXRC</title></head><body>");
                    sb.Append("No projects associated with your username.<br/>");
                    sb.Append("</body><html>");
                    wtr.WriteLine(sb.ToString());
                    wtr.Flush();
                    _contentLength = GetContentLength(context);
                    wtr.Close();
                }
            }
        }

        private void RenderHtmlProject(ControllerContext context, Guid ownerID, string projectName)
        {
            List<FolderDTO> folderList = _dal.ListFolders(context, ownerID, projectName);
            if (folderList != null && folderList.Count() > 0)
            {
                using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
                {
                    StringBuilder sb = new StringBuilder();
                    sb.Append("<html><head><title>CodeXRC</title></head><body>");
                    sb.AppendFormat("Contents of project {0}:<br/><ul>", projectName);
                    if (folderList != null && folderList.Count() > 0)
                    {
                        foreach (FolderDTO item in folderList)
                        {
                            sb.AppendFormat("<li><a href='{0}'>{1}</a>", item.Link, item.DisplayName);
                            if (item.Folders != null && item.Folders.Count() > 0)
                            {
                                sb.Append("<ul>");
                                RenderChildFolders(ref sb, projectName, item.Folders);
                                sb.Append("</ul>");
                            }
                            if (item.Files != null && item.Files.Count() > 0)
                            {
                                sb.Append("<ul>");
                                RenderChildFiles(ref sb, projectName, item.DisplayName, item.Files);
                                sb.Append("</ul>");
                            }
                            sb.Append("</li>");
                        }
                    }
                    sb.Append("</ul></body><html>");
                    wtr.WriteLine(sb.ToString());
                    wtr.Flush();
                    _contentLength = GetContentLength(context);
                    wtr.Close();
                }
            }
            else
            {
                // No folders associated with project
                using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
                {
                    StringBuilder sb = new StringBuilder();
                    sb.Append("<html><head><title>CodeXRC</title></head><body>");
                    sb.AppendFormat("No folders associated with project {0}:<br/>", projectName);
                    sb.Append("</body><html>");
                    wtr.WriteLine(sb.ToString());
                    wtr.Flush();
                    _contentLength = GetContentLength(context);
                    wtr.Close();
                }
            }
        }

        private void RenderHtmlFolder(ControllerContext context, Guid ownerID, string projectName, string folderName)
        {
            List<FolderDTO> folderList = _dal.ListFoldersByFolder(context, ownerID, projectName, folderName);
            List<FileDTO> fileList = _dal.ListFiles(context, ownerID, projectName, folderName);
            if ((folderList != null && folderList.Count() > 0) ||
                (fileList != null && fileList.Count() > 0))
            {
                using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
                {
                    StringBuilder sb = new StringBuilder();
                    sb.Append("<html><head><title>CodeXRC</title></head><body>");
                    sb.AppendFormat("Contents of folder {0}:<br/><ul>", folderName);

                    // Folders first
                    if (folderList != null && folderList.Count() > 0)
                    {
                        foreach (FolderDTO item in folderList)
                        {
                            sb.AppendFormat("<li><a href='{0}'>{1}</a>", item.Link, item.DisplayName);
                            if (item.Folders != null && item.Folders.Count() > 0)
                            {
                                sb.Append("<ul>");
                                RenderChildFolders(ref sb, projectName, item.Folders);
                                sb.Append("</ul>");
                            }
                            if (item.Files != null && item.Files.Count() > 0)
                            {
                                sb.Append("<ul>");
                                RenderChildFiles(ref sb, projectName, item.DisplayName, item.Files);
                                sb.Append("</ul>");
                            }
                            sb.Append("</li>");
                        }
                    }

                    // Then files
                    if (fileList != null && fileList.Count() > 0)
                    {
                        foreach (FileDTO item in fileList)
                        {
                            sb.AppendFormat("<li><a href='{0}'>{1}</a></li>", item.Link, item.DisplayName);
                        }
                    }

                    // Finish it off
                    sb.Append("</ul></body><html>");
                    wtr.WriteLine(sb.ToString());
                    wtr.Flush();
                    _contentLength = GetContentLength(context);
                    wtr.Close();
                }
            }
            else
            {
                // No child folders associated with this folder
                using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
                {
                    StringBuilder sb = new StringBuilder();
                    sb.Append("<html><head><title>CodeXRC</title></head><body>");
                    sb.AppendFormat("No child folders or files associated with folder {0}:<br/>", projectName);
                    sb.Append("</body><html>");
                    wtr.WriteLine(sb.ToString());
                    wtr.Flush();
                    _contentLength = GetContentLength(context);
                    wtr.Close();
                }
            }
        }

        private void RenderHtmlFileByType(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileExtension)
        {
            FileDTO[] files = _dal.GetFileByType(context, ownerID, projectName, folderName, fileExtension);
            context.HttpContext.Response.ContentType = "text/html";
            using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("<html><head><title>CodeXRC</title></head><body>");
                sb.AppendFormat("Files of type {0}:<br/><ul>", fileExtension);
                foreach (FileDTO file in files)
                {
                    // Append this file
                    sb.AppendFormat("<li><a href='{0}'>{1}</a></li>", file.Link, file.DisplayName);
                }
                sb.Append("</ul></body><html>");
                wtr.WriteLine(sb.ToString());
                wtr.Flush();
                _contentLength = GetContentLength(context);
                wtr.Close();
            }
        }

        private void RenderHtmlFileByName(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileExtension, string fileName)
        {
            FileDTO file = _dal.GetFile(context, ownerID, projectName, folderName, fileName + "." + fileExtension);

            // Convert the contents as binary
            file.Contents = _aux.RenderBinaryContent(context, file);

            context.HttpContext.Response.ContentType = "text/plain";
            System.IO.StreamReader fileReader = System.IO.File.OpenText(file.LocalStorage + file.DisplayName);
            using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("<html><head><title>CodeXRC</title></head><body>");
                sb.AppendFormat("Contents for file {0}:<br/><div>", fileName);
                wtr.WriteLine(sb.ToString());

                wtr.WriteLine(file.Contents);

                sb = new StringBuilder();
                sb.Append("</div></body><html>");
                wtr.WriteLine(sb.ToString());
                wtr.Flush();
                _contentLength = GetContentLength(context);
                wtr.Close();
            }
        }

        private void RenderChildFolders(ref StringBuilder sb, string projectName, FolderDTO[] folders)
        {
            if (folders == null) return;
            foreach (FolderDTO item in folders)
            {
                sb.AppendFormat("<li><a href='{0}'>{1}</a>", item.Link, item.DisplayName);
                if (item.Folders != null && item.Folders.Count() > 0)
                {
                    sb.Append("<ul>");
                    RenderChildFolders(ref sb, projectName, item.Folders);
                    sb.Append("</ul>");
                }
                if (item.Files != null && item.Files.Count() > 0)
                {
                    sb.Append("<ul>");
                    RenderChildFiles(ref sb, projectName, item.DisplayName, item.Files);
                    sb.Append("</ul>");
                }
                sb.Append("</li>");
            }
        }

        private void RenderChildFiles(ref StringBuilder sb, string projectName, string folderName, FileDTO[] files)
        {
            foreach (FileDTO item in files)
            {
                sb.AppendFormat("<li><a href='{0}'>{1}</a> (file)</li>", item.Link, item.DisplayName);
            }
        }
    }
}
