﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using System.Web.UI;
using System.IO;
using System.Text;
using System.Runtime.Serialization.Json;

using AspNetMvcRestService.Security;

namespace AspNetMvcRestService.Models
{
    public partial class CodeXRCGetService
    {
        private void RenderJson(ControllerContext context)
        {
            // Pull the user's ID. This works here since we're using the SQL-based
            // membership provider.
            MembershipUser user = Membership.GetUser(context.HttpContext.User.Identity.Name);
            Guid ownerID = (Guid)user.ProviderUserKey;

            context.HttpContext.Response.ContentType = "application/json";
            switch (_directives.Count())
            {
                case 0:
                default:
                    // List all projects for user:
                    // http://{host}/{vdir}/CodeXRC
                    RenderJsonProjectList(context, ownerID);
                    break;

                case 1:
                    // List specific project for user:
                    // http://{host}/{vdir}/CodeXRC/{project}
                    RenderJsonProject(context, ownerID, _directives[0]);
                    break;

                case 2:
                    // List specific folder for user:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}
                    RenderJsonFolder(context, ownerID, _directives[0], _directives[1]);
                    break;

                case 3:
                    // List files by extension for user:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}/{file extension}
                    RenderJsonFilesByType(context, ownerID, _directives[0], _directives[1], _directives[2]);
                    break;

                case 4:
                    // Return specific file to user:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}/{file extension}/{file name}
                    RenderJsonFileByName(context, ownerID, _directives[0], _directives[1], _directives[2], _directives[3]);
                    break;
            }

            // Assign the content length, but only if HEAD.
            Stream stream = context.HttpContext.Items["OutputStream"] as Stream;
            if (stream != null && stream.CanSeek)
            {
                context.HttpContext.Response.Headers["Content-Length"] = GetContentLength(context).ToString();
            }
        }

        private void RenderJsonProjectList(ControllerContext context, Guid ownerID)
        {
            // Pull the project list
            List<ProjectDTO> projectList = _dal.ListProjects(context, ownerID);

            // Create a serializer
            DataContractJsonSerializer serializer =
                new DataContractJsonSerializer(typeof(ProjectDTO[]));

            // Check to see if there are any projects
            if (projectList != null && projectList.Count() > 0)
            {
                // Serialize the project XML.
                serializer.WriteObject((Stream)context.HttpContext.Items["OutputStream"], projectList.ToArray());
            }
            else
            {
                // No projects to show
                ProjectDTO[] emptyList = new ProjectDTO[1];
                emptyList[0] = new ProjectDTO();

                // Serialize the project XML.
                serializer.WriteObject((Stream)context.HttpContext.Items["OutputStream"], emptyList);
            }
        }

        private void RenderJsonProject(ControllerContext context, Guid ownerID, string projectName)
        {
            // Pull the folder list
            List<FolderDTO> folderList = _dal.ListFolders(context, ownerID, projectName);

            // Create a serializer
            DataContractJsonSerializer serializer =
                new DataContractJsonSerializer(typeof(FolderDTO[]));

            // Check to see if there are any folders
            if (folderList != null && folderList.Count() > 0)
            {
                // Serialize the project XML.
                serializer.WriteObject((Stream)context.HttpContext.Items["OutputStream"], folderList.ToArray());
            }
            else
            {
                // No folders to show
                FolderDTO[] emptyList = new FolderDTO[1];
                emptyList[0] = new FolderDTO();

                // Serialize the folder XML.
                serializer.WriteObject((Stream)context.HttpContext.Items["OutputStream"], emptyList);
            }
        }

        private void RenderJsonFolder(ControllerContext context, Guid ownerID, string projectName, string folderName)
        {
            // Pull this folder
            FolderDTO folder = _dal.GetFolder(context, ownerID, projectName, folderName);

            // Create a serializer
            DataContractJsonSerializer serializer =
                new DataContractJsonSerializer(typeof(FolderDTO));

            // Check to see if there are any folders
            if (folder != null)
            {
                // Pull the child folder list
                List<FolderDTO> folderList = _dal.ListFoldersByFolder(context, ownerID, projectName, folderName);
                if (folderList != null && folderList.Count() > 0)
                {
                    folder.Folders = folderList.ToArray();
                }

                // Pull the file list
                List<FileDTO> fileList = _dal.ListFiles(context, ownerID, projectName, folderName);
                if (fileList != null && fileList.Count() > 0)
                {
                    folder.Files = fileList.ToArray();
                }

                // Serialize the folder list XML
                serializer.WriteObject((Stream)context.HttpContext.Items["OutputStream"], folder);
            }
            else
            {
                // No folder to show
                FolderDTO[] emptyList = new FolderDTO[1];
                emptyList[0] = new FolderDTO();

                // Serialize the folder XML
                serializer.WriteObject((Stream)context.HttpContext.Items["OutputStream"], emptyList);
            }
        }

        private void RenderJsonFilesByType(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileExtension)
        {
            FileDTO[] fileList = _dal.GetFileByType(context, ownerID, projectName, folderName, fileExtension);

            // Create a serializer
            DataContractJsonSerializer serializer =
                new DataContractJsonSerializer(typeof(FileDTO[]));

            // Check to see if there are any files
            if (fileList != null && fileList.Count() > 0)
            {
                // Serialize the file list JSON
                serializer.WriteObject((Stream)context.HttpContext.Items["OutputStream"], fileList.ToArray());
            }
            else
            {
                // No files to show
                FolderDTO[] emptyList = new FolderDTO[1];
                emptyList[0] = new FolderDTO();

                // Serialize the file JSON
                serializer.WriteObject((Stream)context.HttpContext.Items["OutputStream"], emptyList);
            }
        }

        private void RenderJsonFileByName(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileExtension, string fileName)
        {
            FileDTO file = _dal.GetFile(context, ownerID, projectName, folderName, fileName + "." + fileExtension);

            // Check to see if there are any files
            if (file != null)
            {
                // Convert the contents as binary
                file.Contents = _aux.RenderBinaryContent(context, file);
            }
            else
            {
                // No file to show
                file = new FileDTO();
            }

            // Create a serializer
            DataContractJsonSerializer serializer =
                new DataContractJsonSerializer(typeof(FileDTO));

            // Serialize the project XML.
            serializer.WriteObject((Stream)context.HttpContext.Items["OutputStream"], file);
        }
    }
}
