﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Xml;

namespace CodeXRCClient
{
    public partial class Form1 : Form
    {
        string _username = String.Empty;
        string _password = String.Empty;
        bool _rememberMe = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Select default content type and method
            cmbContentType.SelectedItem = cmbContentType.Items[Properties.Settings.Default.ContentType];
            cmbMethod.SelectedItem = cmbMethod.Items[Properties.Settings.Default.Method];

            // Pull up the saved URI
            tbServiceUri.Text = Properties.Settings.Default.ServiceUri;
        }

        private void cmdExecute_Click(object sender, EventArgs e)
        {
            // Check for a URI...we can't let the user progress without
            // one...
            if (String.IsNullOrEmpty(tbServiceUri.Text))
            {
                // No URI...
                MessageBox.Show("You must provide a URI before executing the service call.",
                    "URI Required", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            LocalProjectDTO newProject = null;
            if (cmbMethod.SelectedIndex == 2)
            {
                // Verify XML content type...
                if (cmbContentType.SelectedIndex != 1)
                {
                    // Can only do XML...
                    MessageBox.Show("The service only accepts XML for HTTP PUT method.",
                        "Incorrect content type", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                // We're executing a PUT, so we need a representation
                // to send to the service. We'll ask for the information
                // before communicating with the server.
                ProjectCreationForm newProjectDlg = new ProjectCreationForm();

                // Show the dialog
                if (newProjectDlg.ShowDialog() == DialogResult.OK)
                {
                    // Pull completed project DTO.
                    newProject = newProjectDlg.Project;
                }
                else
                {
                    // User canceled, so exit method
                    return;
                }
            }

            // Persist the service URI, content type, and method
            Properties.Settings.Default.ServiceUri = tbServiceUri.Text;
            Properties.Settings.Default.ContentType = cmbContentType.SelectedIndex;
            Properties.Settings.Default.Method = cmbMethod.SelectedIndex;
            Properties.Settings.Default.Save();

            Int32 attempts = 3;
            while (attempts > 0)
            {
                try
                {
                    // Change cursor
                    this.Cursor = Cursors.WaitCursor;

                    // Check for authentication
                    string authorizedHeader = String.Empty;
                    if (!String.IsNullOrEmpty(this._username) && !String.IsNullOrEmpty(this._password))
                    {
                        // Create the header
                        string decoded = String.Format("{0}:{1}", this._username, this._password);
                        string encoded = Convert.ToBase64String(Encoding.ASCII.GetBytes(decoded));
                        authorizedHeader = String.Format("Authorization: BASIC {0}", encoded);
                    }

                    // Create the request
                    Uri uri = new Uri(tbServiceUri.Text);
                    HttpWebRequest request = WebRequest.Create(uri) as HttpWebRequest;
                    request.Method = cmbMethod.Items[cmbMethod.SelectedIndex].ToString();
                    switch (cmbContentType.SelectedIndex)
                    {
                        case 0:
                        default:
                            request.Accept = "text/html";
                            break;

                        case 1:
                            request.Accept = "text/xml";
                            break;

                        case 2:
                            request.Accept = "application/json";
                            break;
                    }
                    if (!String.IsNullOrEmpty(authorizedHeader))
                    {
                        request.Headers.Add(authorizedHeader);
                    }

                    // Send any data
                    switch (cmbMethod.SelectedIndex)
                    {
                        case 0: // HEAD, no data
                        case 1: // GET, no data
                        case 3: // DELETE, no data
                            // No data to send, just ask for response
                            break;

                        case 2: // PUT
                            PushProjectXml(request, newProject);
                            break;
                    }

                    // Call for the response
                    HttpWebResponse response = request.GetResponse() as HttpWebResponse;

                    // If we made a HEAD call, display the headers, else the content
                    if (cmbMethod.SelectedIndex == 0)
                    {
                        // Build response header output
                        StringBuilder sb = new StringBuilder();
                        sb.AppendFormat("HTTP/{0} {1} {2}\r\n", response.ProtocolVersion, (Int32)response.StatusCode, response.StatusDescription);

                        // Retrieve the response packet headers
                        for (Int32 i = response.Headers.Count - 1; i >= 0; i--)
                        {
                            sb.AppendFormat("{0}: {1}\r\n", response.Headers.GetKey(i), response.Headers[i]);
                        }

                        // Assign text
                        tbResults.Text = sb.ToString();
                    }
                    else
                    {
                        // Retrieve content
                        string responseContent = String.Empty;
                        using (StreamReader rdr = new StreamReader(response.GetResponseStream()))
                        {
                            responseContent = rdr.ReadToEnd();
                        }

                        tbResults.Text = responseContent;
                    }

                    // Clear username/password if we're to forget
                    if (!this._rememberMe)
                    {
                        this._username = String.Empty;
                        this._password = String.Empty;
                    }

                    // Exit the loop
                    break;
                }
                catch (WebException ex)
                {
                    // Check for authentication
                    HttpWebResponse response = ex.Response as HttpWebResponse;
                    if (response.StatusCode == HttpStatusCode.Unauthorized)
                    {
                        // Pop up new username/password dialog box
                        ClientAuthenticationForm authForm = new ClientAuthenticationForm();
                        if (authForm.ShowDialog() == DialogResult.OK)
                        {
                            // Save username/password...
                            this._username = authForm.Username;
                            this._password = authForm.Password;

                            // Save "remember me" setting
                            this._rememberMe = authForm.RememberMeChecked;
                        }
                        else
                        {
                            // User canceled...
                            break;
                        }
                    }
                    else
                    {
                        // Pop up a messagebox showing the error...
                        MessageBox.Show(ex.Message, "Error Invoking RESTful Service",
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                        tbResults.Text = ex.ToString();
                        break;
                    }
                }
                catch (Exception ex)
                {
                    // Just display the message
                    MessageBox.Show(ex.Message, "Internal Application Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                finally
                {
                    // Change cursor back
                    this.Cursor = Cursors.Default;
                }

                // Decrease attempt number
                --attempts;
            }
        }

        private void cmdExit_Click(object sender, EventArgs e)
        {
            // Exit the app
            Properties.Settings.Default.Save();
            Application.Exit();
        }

        private void PushProjectXml(HttpWebRequest request, LocalProjectDTO project)
        {
            Stream requestStream = request.GetRequestStream();
            DataContractSerializer projectSerializer =
                new DataContractSerializer(typeof(LocalProjectDTO));
            projectSerializer.WriteObject(
                requestStream, project);

            // Clear the stream;
            requestStream.Flush();
            requestStream.Close();

            // Set the content type
            request.ContentType = "text/xml";
        }

        private void PushProjectJson(HttpWebRequest request, LocalProjectDTO project)
        {
            Stream requestStream = request.GetRequestStream();
            DataContractJsonSerializer projectSerializer =
                new DataContractJsonSerializer(typeof(LocalProjectDTO));
            projectSerializer.WriteObject(
                requestStream, project);

            // Clear the stream;
            requestStream.Flush();
            requestStream.Close();

            // Set the content type
            request.ContentType = "application/json";
        }
    }
}
