﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CodeXRCClient
{
    public partial class ProjectCreationForm : Form
    {
        private const string DeleteNodeTipText = "To remove, select and then press delete.";

        private LocalProjectDTO _newProject = null;

        public ProjectCreationForm()
        {
            InitializeComponent();
        }

        public LocalProjectDTO Project
        {
            get { return _newProject; }
        }

        private void cmdAddDir_Click(object sender, EventArgs e)
        {
            // Bring up the folder selection dialog
            FolderBrowser folderDlg = new FolderBrowser();
            folderDlg.OnlyFilesystem = true;
            if (folderDlg.ShowDialog() == DialogResult.OK)
            {
                // Fill tree control...
                this.Cursor = Cursors.WaitCursor;
                FillDirectoryTree(folderDlg.DirectoryPath);
                this.Cursor = Cursors.Arrow;

                // Enable OK button
                if (tvFiles.Nodes.Count > 0)
                {
                    cmdOK.Enabled = true;
                }
            }
        }

        private void cmdAddFile_Click(object sender, EventArgs e)
        {
            // Activate the OpenFilesDialog and ask
            // for files to add.
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.CheckFileExists = true;
            ofd.CheckPathExists = true;
            ofd.Filter = "All files (*.*)|*.*";
            ofd.Multiselect = true;
            ofd.Title = "Project File Selection";
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                // Fill tree control...
                this.Cursor = Cursors.WaitCursor;
                FillFileTree(ofd.FileNames);
                this.Cursor = Cursors.Arrow;

                // Enable OK button
                if (tvFiles.Nodes.Count > 0)
                {
                    cmdOK.Enabled = true;
                }
            }
        }

        private void cmdOK_Click(object sender, EventArgs e)
        {
            // Create the appropriate XML
            if (tvFiles.Nodes.Count > 0)
            {
                // Create the XML to send
                CreateProject();

                // Close
                this.DialogResult = DialogResult.OK;
            }
            else
            {
                // Just close
                this.DialogResult = DialogResult.Cancel;
            }
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            // Just close
            this.DialogResult = DialogResult.Cancel;
        }

        private void tbProjectName_TextChanged(object sender, EventArgs e)
        {
            // Enable/disable directory/file addition buttons
            cmdAddDir.Enabled = cmdAddFile.Enabled = !String.IsNullOrEmpty(tbProjectName.Text);
        }

        private void tvFiles_AfterExpand(object sender, TreeViewEventArgs e)
        {
            if (e.Node.ImageIndex == 1)
            {
                // Set the node's image to an open folder
                e.Node.ImageIndex = 2;
            }
        }

        private void tvFiles_AfterCollapse(object sender, TreeViewEventArgs e)
        {
            if (e.Node.ImageIndex == 2)
            {
                // Set the node's image to an closed folder
                e.Node.ImageIndex = 1;
            }
        }

        private void tvFiles_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node.ImageIndex == 1 && e.Node.IsExpanded)
            {
                // Set the node's image to an open folder if
                // the node is expanded
                e.Node.ImageIndex = 2;
            }
        }

        private void tvFiles_KeyUp(object sender, KeyEventArgs e)
        {
            // Check for the delete key
            if (e.KeyCode == Keys.Delete && tvFiles.SelectedNode != null)
            {
                // Remove the node
                tvFiles.Nodes.Remove(tvFiles.SelectedNode);
            }
        }

        private void FillDirectoryTree(string folder)
        {
            // Disable updates
            tvFiles.SuspendLayout();

            // Remove any existing nodes
            tvFiles.Nodes.Clear();

            // Create a root node
            TreeNode root = new TreeNode(tbProjectName.Text, 0, 0);
            tvFiles.Nodes.Add(root);

            // Mark as folder
            root.Tag = null;

            // Add tooltip
            root.ToolTipText = DeleteNodeTipText;

            // Locate the directory
            DirectoryInfo dir = new DirectoryInfo(folder);

            // Start the recursion
            AddFolder(dir, root);

            // Enable updates
            tvFiles.ResumeLayout();
        }

        private void AddFolder(DirectoryInfo folder, TreeNode parent)
        {
            try
            {
                // Check for child folders
                DirectoryInfo[] childFolders = folder.GetDirectories();
                if (childFolders.Count() > 0)
                {
                    // Iterate through each child folder
                    TreeNode dirNode = null;
                    foreach (DirectoryInfo dir in childFolders)
                    {
                        // Create a node for this directory
                        dirNode = new TreeNode(dir.Name, 1, 1);
                        parent.Nodes.Add(dirNode);

                        // Mark as folder
                        dirNode.Tag = null;

                        // Add tooltip
                        dirNode.ToolTipText = DeleteNodeTipText;

                        // Iterate
                        AddFolder(dir, dirNode);
                    }
                }

                // Get the files
                FileInfo[] files = folder.GetFiles();

                // Add the files to the parent node
                foreach (FileInfo file in files)
                {
                    // Add this file
                    AddFile(file, parent);
                }
            }
            catch
            {
                // If a file was locked, we'll sustain a permission exception. Just
                // ignore such exceptions and move on.
            }
        }

        private void FillFileTree(string[] fileNames)
        {
            // Disable updates
            tvFiles.SuspendLayout();

            // Remove any existing nodes
            tvFiles.Nodes.Clear();

            // Create a root node
            TreeNode root = new TreeNode(tbProjectName.Text, 0, 0);
            tvFiles.Nodes.Add(root);

            // Mark as folder
            root.Tag = null;

            // Add tooltip
            root.ToolTipText = DeleteNodeTipText;

            // If we have files, add them
            if (fileNames.Length > 0)
            {
                // We have to have a parent directory name,
                // but when selecting files, you don't select
                // a directory. So for lack of a better choice,
                // we'll use the project again.
                TreeNode parentFolder = new TreeNode(tbProjectName.Text, 2, 2);
                root.Nodes.Add(parentFolder);

                // Start the recursion
                foreach (string fileName in fileNames)
                {
                    // Add this file
                    FileInfo file = new FileInfo(fileName);
                    AddFile(file, parentFolder);
                }
            }

            // Enable updates
            tvFiles.ResumeLayout();
        }

        private void AddFile(FileInfo file, TreeNode parent)
        {
            try
            {
                TreeNode fileNode = null;
                switch (file.Extension.ToLower())
                {
                    case ".bmp":
                    case ".png":
                    case ".jpg":
                    case ".gif":
                    case ".ico":
                        // Image file types we understand
                        fileNode = new TreeNode(file.Name, 4, 4);
                        parent.Nodes.Add(fileNode);
                        break;

                    default:
                        // Assumed to be a text file of some kind
                        fileNode = new TreeNode(file.Name, 3, 3);
                        parent.Nodes.Add(fileNode);
                        break;
                }

                // Save the full filename
                fileNode.Tag = file.FullName;

                // Add tooltip
                fileNode.ToolTipText = DeleteNodeTipText;
            }
            catch
            {
                // If a file was locked, we'll sustain a permission exception. Just
                // ignore such exceptions and move on.
            }
        }

        private void CreateProject()
        {
            // Follow the tree nodes and build the DTO
            // collections.
            _newProject = new LocalProjectDTO();
            _newProject.ProjectID = "-1";
            _newProject.DisplayName = tbProjectName.Text;

            // Create a "solution items" folder for any files
            // directly attributed to the project
            LocalFolderDTO solutionFolder = null;

            // Add folders and files
            foreach (TreeNode child in tvFiles.Nodes[0].Nodes)
            {
                if (child.Tag == null)
                {
                    // Recurse our child folders
                    CreateFolders(child, _newProject.FoldersList);
                }
                else
                {
                    // Create a "solution items" folder
                    if (solutionFolder == null)
                    {
                        solutionFolder = new LocalFolderDTO();
                        solutionFolder.DisplayName = "Solution Items";
                        solutionFolder.FolderID = "-1";
                        solutionFolder.ParentFolderID = "-1";
                        solutionFolder.ProjectID = "-1";

                        // Add it to the project
                        _newProject.FoldersList.Add(solutionFolder);
                    }

                    // Create a file DTO
                    LocalFileDTO file = new LocalFileDTO();
                    file.DisplayName = child.Text;
                    file.FileID = "-1";
                    file.FolderID = "-1";
                    file.ProjectID = "-1";
                    file.Contents = RenderBinaryContent((string)child.Tag);

                    // Add our files directly
                    solutionFolder.FilesList.Add(file);
                }
            }
        }

        private void CreateFolders(TreeNode me, List<LocalFolderDTO> parentList)
        {
            // Create a new folder DTO for ourselves
            LocalFolderDTO folder = new LocalFolderDTO();
            folder.DisplayName = me.Text;
            folder.FolderID = "-1";
            folder.ParentFolderID = "-1";
            folder.ProjectID = "-1";

            // Add it to our parent list
            parentList.Add(folder);
            
            // Add folders and files
            foreach (TreeNode child in me.Nodes)
            {
                if (child.Tag == null)
                {
                    // Recurse our child folders
                    CreateFolders(child, folder.FoldersList);
                }
                else
                {
                    // Create a file DTO
                    LocalFileDTO file = new LocalFileDTO();
                    file.DisplayName = child.Text;
                    file.FileID = "-1";
                    file.FolderID = "-1";
                    file.ProjectID = "-1";
                    file.Contents = RenderBinaryContent((string)child.Tag);

                    // Add our files directly
                    folder.FilesList.Add(file);
                }
            }
        }

        public string RenderBinaryContent(string fileName)
        {
            // We'll need to open the file and convert the contents to Base64
            // before assigning the contents to the DTO.
            string contents = String.Empty;
            if (System.IO.File.Exists(fileName))
            {
                // Open the file stream for conversion
                using (FileStream fstm = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read))
                {
                    // Create a temporary stream to hold the contents
                    MemoryStream temp = new MemoryStream();

                    // Copy the contents of the file into the temporary stream
                    CopyStream(fstm, temp);

                    // Convert the contents and close the temporary stream
                    contents = Convert.ToBase64String(temp.ToArray());
                    temp.Close();
                }
            }

            return contents;
        }

        public void CopyStream(Stream input, Stream output)
        {
            byte[] buffer = new byte[0x1000];
            Int32 bytesRead = 0;
            while ((bytesRead = input.Read(buffer, 0, buffer.Length)) > 0)
                output.Write(buffer, 0, bytesRead);
        }
    }
}
