﻿// Effective REST Services via .NET, ISBN: 0321613406
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!
using System;
using System.ComponentModel;
using System.Globalization;
using System.Security.Permissions;
using System.ServiceModel.Syndication;
using System.Web;
using System.Web.Caching;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;

namespace ScottSeely.Web
{
    [DefaultProperty("FeedUri"),
    ToolboxData("<{0}:FeedSummary runat=\"server\" ></{0}:FeedSummary"),
    AspNetHostingPermission(SecurityAction.Demand,
        Level = AspNetHostingPermissionLevel.Minimal),
    AspNetHostingPermission(SecurityAction.InheritanceDemand,
        Level = AspNetHostingPermissionLevel.Minimal)]
    public class FeedSummary : WebControl
    {
        const string DataCategory = "Data";
        const string CacheCategory = "Cache";

        [Bindable(true),
        Category(DataCategory),
        DefaultValue("")]
        public Uri FeedUri { get; set; }

        [Bindable(true),
        Category(DataCategory),
        DefaultValue(3)]
        public int NumberOfFeedItems { get; set; }

        [Bindable(true),
        Category(DataCategory),
        DefaultValue(FeedDisplayStyle.TitleOnly)]
        public FeedDisplayStyle FeedDisplayStyle { get; set; }

        [Bindable(true),
                Category(CacheCategory),
                DefaultValue(true)]
        public bool CacheFeedItems { get; set; }

        [Bindable(true),
                        Category(CacheCategory),
                        DefaultValue(true)]
        public int MinutesToCacheFeedItems { get; set; }

        static string CreateFeedKey(Uri feedUri)
        {
            return string.Format(CultureInfo.InvariantCulture,
                "ScottSeely.Web.Feed.{0}", feedUri.ToString());
        }

        static SyndicationFeed RetrieveFeed(Uri feedUri, bool addToCache, int minutesToCacheItems)
        {
            SyndicationFeed retval = null;
            if (HttpContext.Current != null)
            {
                retval = HttpContext.Current.Cache[CreateFeedKey(feedUri)] as SyndicationFeed;
            }
            if (retval == null)
            {
                retval = SyndicationFeed.Load(new XmlTextReader(feedUri.ToString()));
                if (addToCache)
                {
                    if (HttpContext.Current != null)
                    {
                        HttpContext.Current.Cache.Add(CreateFeedKey(feedUri), retval,
                            null, DateTime.Now.AddMinutes(minutesToCacheItems), Cache.NoSlidingExpiration,
                            CacheItemPriority.Normal, null);
                    }
                }
            }
            return retval;
        }

        protected override void Render(HtmlTextWriter writer)
        {
            base.RenderContents(writer);
            if (CssClass != null)
            {
                writer.AddAttribute("class", CssClass);
            }
            writer.RenderBeginTag(HtmlTextWriterTag.Div);
            SyndicationFeed feed = RetrieveFeed(this.FeedUri, this.CacheFeedItems, this.MinutesToCacheFeedItems);

            writer.RenderBeginTag(HtmlTextWriterTag.Ul);
            foreach (SyndicationItem feedItem in feed.Items)
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Li);
                writer.AddAttribute("href", feedItem.Id);
                writer.RenderBeginTag(HtmlTextWriterTag.A);
                writer.Write(feedItem.Title.Text);
                writer.RenderEndTag();
                writer.RenderBeginTag(HtmlTextWriterTag.I);
                writer.Write(feedItem.PublishDate.Date.ToShortDateString());
                writer.RenderEndTag();
                switch (this.FeedDisplayStyle)
                {
                    case FeedDisplayStyle.TitleOnly:
                        break;
                    case FeedDisplayStyle.TitleAndShortDescription:
                        if (feedItem.Summary != null)
                        {
                            const string endParagraph = "</p>";
                            int endOfParagraphIndex = feedItem.Summary.Text.IndexOf(endParagraph);
                            string shortDescription = feedItem.Summary.Text;
                            if (endOfParagraphIndex > 0)
                            {
                                shortDescription = feedItem.Summary.Text.Substring(0,
                                feedItem.Summary.Text.IndexOf(endParagraph) + endParagraph.Length);
                            }
                            AddParagraph(writer, shortDescription);
                            writer.RenderBeginTag(HtmlTextWriterTag.I);
                            writer.AddAttribute("href", feedItem.Id);
                            writer.RenderBeginTag(HtmlTextWriterTag.A);
                            writer.Write("more");
                            writer.RenderEndTag();
                            writer.RenderEndTag();
                        }
                        break;
                    case FeedDisplayStyle.TitleAndFullDescription:
                        if (feedItem.Summary != null)
                        {
                            AddParagraph(writer, feedItem.Summary.Text);
                        }
                        break;
                }
                writer.RenderBeginTag(HtmlTextWriterTag.Br);
                writer.RenderEndTag();
                writer.RenderEndTag();
            }
            writer.RenderEndTag();
            writer.RenderEndTag();
        }

        static void AddParagraph(HtmlTextWriter writer, string text)
        {
            writer.RenderBeginTag(HtmlTextWriterTag.P);
            writer.Write(text);
            writer.RenderEndTag();
        }
    }
}
