﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Microsoft.ServiceBus.Web;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.ServiceModel.Channels;
using System.ServiceModel.Syndication;
using System.Text;
using System.IO;
using System.Runtime.Serialization;

using CommentSvc.Model;

namespace CommentSvc
{
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single,
                     ConcurrencyMode = ConcurrencyMode.Multiple,
                     AddressFilterMode = AddressFilterMode.Prefix)]
    public class CommentService : ICommentService
    {
        #region Constants

        private const string FeedTitle = "Comments to Effective REST Services via .NET";
        private const string FeedDescription = "Reader comments, inputs, and suggestions.";
        private const string DefaultFeedItemUri = "";
        private const string DefaultFeedItemTitle = "Comment to Effective REST Services via .NET";
        private const string DefaultFeedItemName = "Anonymous";
        private const string DefaultFeedItemSummaryFormat = "Comment from {0}";
        private const string AtomContentType = "application/atom+xml";
        private const string XmlContentType = "text/xml";
        private const string DeserializationFault = "You must provide a properly formatted, XML-based comment item in order to insert new comments into the list.";

        #endregion

        #region Fields

        CommentModel _model = new CommentModel();
        DataContractSerializer _commentXmlItemSerializer =
            new DataContractSerializer(typeof(CommentDTO));

        #endregion

        public Message GetAllComments()
        {
            // Tweak the URI if necessary...we need for it to end with '/'
            // so we can generate proper item URIs.
            Uri  baseUri = OperationContext.Current.IncomingMessageProperties.Via;
            if (!baseUri.AbsoluteUri.EndsWith("/"))
            {
                baseUri = new Uri(baseUri.AbsoluteUri + "/");
            }

            // Retrieve the formatter
            Atom10FeedFormatter formatter = GetAllCommentsFormatter(baseUri);

            // Create the response message
            Message response = Message.CreateMessage(OperationContext.Current.IncomingMessageVersion, "GETALLRESPONSE", formatter);

            // Return the Atom feed
            WebOperationContext.Current.OutgoingResponse.ContentType = AtomContentType;
            return response;
        }

        public Message GetComment(string commentID)
        {
            // Pull in the URI...this is the link to use for this single item
            Uri itemUri = OperationContext.Current.IncomingMessageProperties.Via;

            // Convert the comment ID from string to integer
            Int32 id = 0;
            Int32.TryParse(commentID, out id);

            // If the ID didn't convert, return all comments. Note we can't
            // use GetAllComments because it uses a different reply action
            // when creating the return message.
            Message response = null;
            bool returnAll = false;
            if (id > 0)
            {
                // Valid integer value, so hit the database to see
                // if we have such a comment.
                CommentDTO item = _model.GetComment(id);
                if (item != null)
                {
                    // Generate the link
                    item.Link = itemUri.AbsoluteUri;

                    // Generate the feed
                    List<SyndicationItem> list = new List<SyndicationItem>();
                    list.Add(GetSyndicatedComment(item));
                    SyndicationFeed feed = new SyndicationFeed(FeedTitle, FeedDescription, itemUri, list);
                    Atom10FeedFormatter formatter = new Atom10FeedFormatter(feed);

                    // Create the response message
                    response = Message.CreateMessage(OperationContext.Current.IncomingMessageVersion, "GETRESPONSE", formatter);
                }
                else
                {
                    // Return all
                    returnAll = true;
                }
            }
            else
            {
                // Return all
                returnAll = true;
            }

            // If we need to generate a feed to return all comments,
            // do so now.
            if (returnAll)
            {
                // Strip the invalid comment ID from the incoming URI
                string baseUri = itemUri.AbsoluteUri.Substring(0, itemUri.AbsoluteUri.LastIndexOf('/') + 1);
                Atom10FeedFormatter formatter = GetAllCommentsFormatter(new Uri(baseUri));

                // Create the response message
                response = Message.CreateMessage(OperationContext.Current.IncomingMessageVersion, "GETRESPONSE", formatter);
            }

            // Return the Atom feed
            WebOperationContext.Current.OutgoingResponse.ContentType = AtomContentType;
            return response;
        }

        public Message SaveComment(CommentDTO item)
        {
            // NOTE: this method assumes the content of the comment has been
            // reviewed and approved through some workflow process not shown
            // here. It is not advisable to simply post comments without
            // reviewing them first. Also consider SQL Injection attacks and
            // preventive measures.
            //
            // Make sure we have a DTO
            Message response = null;
            if (item != null)
            {
                // Save the comment
                _model.InsertComment(item);

                // Tweak the URI if necessary...we need for it to end with '/'
                // so we can generate proper item URIs.
                Uri baseUri = OperationContext.Current.IncomingMessageProperties.Via;
                if (!baseUri.AbsoluteUri.EndsWith("/"))
                {
                    baseUri = new Uri(baseUri.AbsoluteUri + "/");
                }

                // Generate the link
                item.Link = String.Format("{0}{1}", baseUri.AbsoluteUri, item.CommentID);

                // Generate the feed
                List<SyndicationItem> list = new List<SyndicationItem>();
                list.Add(GetSyndicatedComment(item));
                SyndicationFeed feed = new SyndicationFeed(FeedTitle, FeedDescription, new Uri(item.Link), list);
                Atom10FeedFormatter formatter = new Atom10FeedFormatter(feed);

                // Create the response message
                response = Message.CreateMessage(OperationContext.Current.IncomingMessageVersion, "POSTRESPONSE", formatter);

                // Assign the usual headers
                WebOperationContext.Current.OutgoingResponse.Location = item.Link;
                WebOperationContext.Current.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.Created;
                WebOperationContext.Current.OutgoingResponse.StatusDescription = "Created";

                // Assign the content type
                WebOperationContext.Current.OutgoingResponse.ContentType = AtomContentType;
            }
            else
            {
                // Problem deserializing...
                WebOperationContext.Current.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.BadRequest;
                WebOperationContext.Current.OutgoingResponse.StatusDescription = "Bad Request";

                // Create the response message
                response = Message.CreateMessage(OperationContext.Current.IncomingMessageVersion, "POSTRESPONSE", (object)DeserializationFault);

                // Assign the content type...WCF will place this in an
                // XML format for us.
                WebOperationContext.Current.OutgoingResponse.ContentType = XmlContentType;
            }

            // Return the Atom feed with the newly-inserted item or the error message
            return response;
        }

        #region Helpers

        private SyndicationItem GetSyndicatedComment(CommentDTO comment)
        {
            // Sanitize the name and e-mail, as these could
            // be null...
            string name = String.IsNullOrEmpty(comment.Name) ? DefaultFeedItemName : comment.Name;
            string email = String.IsNullOrEmpty(comment.EMail) ? String.Empty : comment.EMail;

            // Create a syndication item using the information contained
            // in the comment.
            SyndicationItem item = new SyndicationItem(DefaultFeedItemTitle, comment.Comment, new Uri(comment.Link));
            item.Authors.Add(new SyndicationPerson(email, name, DefaultFeedItemUri));
            item.Summary = new TextSyndicationContent(String.Format(DefaultFeedItemSummaryFormat, name));

            // Return the item
            return item;
        }

        private Atom10FeedFormatter GetAllCommentsFormatter(Uri itemUri)
        {
            // Retrieve all comments
            CommentModel model = new CommentModel();
            CommentDTO[] items = model.GetComments();

            // Generate the feed
            List<SyndicationItem> list = new List<SyndicationItem>();
            foreach (CommentDTO item in items)
            {
                // Generate the link
                item.Link = String.Format("{0}{1}", itemUri.AbsoluteUri, item.CommentID);

                // Add the comment
                list.Add(GetSyndicatedComment(item));
            }
            SyndicationFeed feed = new SyndicationFeed(itemUri.ToString(), FeedTitle, itemUri, list);
            Atom10FeedFormatter formatter = new Atom10FeedFormatter(feed);

            // Return the formatter
            return formatter;
        }

        #endregion
    }
}
