﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.ServiceModel.Description;
using System.Runtime.Serialization;
using Microsoft.ServiceBus;

namespace WpfAzureClient
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        private const string AzureIdentityUriFormat = "https://accesscontrol.windows.net/issuetoken.aspx?u={0}&p={1}";
        private const string IdentityHeader = "X-MS-Identity-Token";

        private string _xIdentityToken = String.Empty;

        public Window1()
        {
            InitializeComponent();
        }

        private void Execute_Click(object sender, RoutedEventArgs e)
        {
            // Check for a URI...we can't let the user progress without
            // one...
            if (String.IsNullOrEmpty(tbServiceUri.Text))
            {
                // No URI...
                MessageBox.Show("You must provide a URI before executing the service call.", "URI Required", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            // Check for a username and password. We'll only need to
            // perform this once...
            if (String.IsNullOrEmpty(_xIdentityToken))
            {
                // Request user's username and password, then query
                // Azure's identity service for a valid authentication
                // token.
                if (!QueryAuthenticationToken())
                {
                    // Exception or user cancelled...
                    return;
                }
            }

            CommentItem newCommentItem = null;
            if (cmbMethod.SelectedIndex == 1)
            {
                // We're executing a POST, so we need a representation
                // to send to the service. We'll ask for the information
                // before communicating with the server.
                NewCommentWindow newCommentDlg = new NewCommentWindow();

                // Show the dialog
                bool? result = newCommentDlg.ShowDialog();
                if (result == true)
                {
                    // Pull comment item...
                    newCommentItem = newCommentDlg.CommentItem;
                }
                else
                {
                    // User canceled, so exit method
                    return;
                }
            }

            try
            {
                // Create the request
                Uri uri = new Uri(tbServiceUri.Text);
                HttpWebRequest request = WebRequest.Create(uri) as HttpWebRequest;
                request.Method = ((ComboBoxItem)cmbMethod.Items[cmbMethod.SelectedIndex]).Tag.ToString();
                request.Headers.Add(IdentityHeader, _xIdentityToken);

                // Send any data
                if (cmbMethod.SelectedIndex == 1)
                {
                    // Insert a new comment
                    PushCommentItem(request, newCommentItem);
                }

                // Call for the response
                HttpWebResponse response = request.GetResponse() as HttpWebResponse;

                // Retrieve content
                string responseContent = String.Empty;
                using (StreamReader rdr = new StreamReader(response.GetResponseStream()))
                {
                    responseContent = rdr.ReadToEnd();
                }
                tbResults.Text = responseContent;
            }
            catch (Exception ex)
            {
                // Pop up a messagebox showing the error...
                MessageBox.Show(ex.Message, "Error Invoking Azure Comments Service", MessageBoxButton.OK, MessageBoxImage.Error);
                tbResults.Text = ex.ToString();
            }
        }

        private void Exit_Click(object sender, RoutedEventArgs e)
        {
            // Exit the app
            Properties.Settings.Default.Save();
            this.Close();
        }

        #region PushCommentItem

        private void PushCommentItem(HttpWebRequest request, CommentItem item)
        {
            Stream requestStream = request.GetRequestStream();
            try
            {
                DataContractSerializer serializer =
                    new DataContractSerializer(typeof(CommentItem));
                serializer.WriteObject(
                    requestStream, item);
            }
            finally
            {
                // Clear the stream;
                requestStream.Flush();
                requestStream.Close();
            }

            // Set the content type
            request.ContentType = "text/xml";
        }

        #endregion

        #region QueryAuthenticationToken

        private bool QueryAuthenticationToken()
        {
            // Assume not authenticated
            bool authenticated = false;

            // First request username/password
            ClientAuthenticationWindow authWindow = new ClientAuthenticationWindow();
            authWindow.Owner = this;
            bool? dialogResult = authWindow.ShowDialog();
            if (dialogResult == true)
            {
                try
                {
                    // Next, query the Azure identity service for
                    // a valid authentication token.
                    Uri uri = new Uri(String.Format(AzureIdentityUriFormat, authWindow.Username, Uri.EscapeDataString(authWindow.Password)));
                    HttpWebRequest request = WebRequest.Create(uri) as HttpWebRequest;
                    request.Method = "GET";

                    // Call for the response
                    HttpWebResponse response = request.GetResponse() as HttpWebResponse;

                    // Read the response
                    using (StreamReader rdr = new StreamReader(response.GetResponseStream()))
                    {
                        // Scrape the token cookie
                        _xIdentityToken = rdr.ReadToEnd();

                        // HACK: Remove extraneous content noted at times from
                        // Access Control Service. Sometimes we obtain a valid
                        // token, and other times we obtain the token with some
                        // HTML attached telling us there was a server-side
                        // error. Since we have a token, slice off the HTML
                        // and ignore it. Note the token is a Base64 string,
                        // so we can safely parse the '>'
                        Int32 htmlStart = _xIdentityToken.IndexOf('<');
                        if (htmlStart > 0)
                            _xIdentityToken = _xIdentityToken.Substring(0, _xIdentityToken.IndexOf('<'));
                    }

                    // With the cookie in hand, indicate we authenticated
                    authenticated = true;
                }
                catch (Exception ex)
                {
                    // Some error..show user
                    MessageBox.Show(String.Format("Error accessing Azure identity service: {0}", ex.ToString()),
                        "Authentication Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }

            return authenticated;
        }

        #endregion
    }
}
